==> merkle-airdrop-0.3.5.tgz: package/src/wrapper.ts <==
import { TransactionEnvelope } from "@saberhq/solana-contrib";
import type { u64 } from "@saberhq/token-utils";
import {
  getATAAddress,
  getOrCreateATA,
  TOKEN_PROGRAM_ID,
  NATIVE_MINT,
  ASSOCIATED_TOKEN_PROGRAM_ID,
} from "@saberhq/token-utils";
import type { PublicKey, TransactionInstruction } from "@solana/web3.js";
import { Keypair, SystemProgram } from "@solana/web3.js";
import { Token } from "@solana/spl-token";

import { findClaimStatusKey, findDistributorKey } from "./pda";
import type { MerkleDistributorSDK } from "./sdk";
import type {
  ClaimArgs,
  ClaimStatus,
  CreateDistributorArgs,
  DistributorData,
  MerkleDistributorProgram,
  PendingDistributor,
} from "./types";
import { toBytes32Array } from "./utils";

export class MerkleDistributorWrapper {
  readonly program: MerkleDistributorProgram;
  readonly key: PublicKey;
  readonly distributorATA: PublicKey;
  data: DistributorData;

  constructor(
    readonly sdk: MerkleDistributorSDK,
    key: PublicKey,
    distributorATA: PublicKey,
    data: DistributorData
  ) {
    this.program = sdk.program;
    this.key = key;
    this.distributorATA = distributorATA;
    this.data = data;
  }

  static async load(
    sdk: MerkleDistributorSDK,
    key: PublicKey
  ): Promise<MerkleDistributorWrapper> {
    const data = await sdk.program.account.merkleDistributor.fetch(key);
    return new MerkleDistributorWrapper(
      sdk,
      key,
      await getATAAddress({ mint: data.mint, owner: key }),
      data
    );
  }

  static async createDistributor(
    args: CreateDistributorArgs
  ): Promise<PendingDistributor> {
    const { root, tokenMint } = args;

    const { sdk } = args;
    const { provider } = sdk;

    const baseKey = args.base ?? Keypair.generate();
    const [distributor, bump] = await findDistributorKey(baseKey.publicKey);

    const ixs: TransactionInstruction[] = [];
    ixs.push(
      sdk.program.instruction.newDistributor(
        bump,
        toBytes32Array(root),
        args.maxTotalClaim,
        args.maxNumNodes,
        {
          accounts: {
            base: baseKey.publicKey,
            distributor,
            mint: tokenMint,
            payer: provider.wallet.publicKey,
            systemProgram: SystemProgram.programId,
          },
        }
      )
    );

    const { address, instruction } = await getOrCreateATA({
      provider,
      mint: tokenMint,
      owner: distributor,
    });
    if (instruction) {
      ixs.push(instruction);
    }

    return {
      base: baseKey.publicKey,
      bump,
      distributor,
      distributorATA: address,
      tx: new TransactionEnvelope(provider, ixs, [baseKey]),
    };
  }

  async claimIX(
    args: ClaimArgs,
    payer: PublicKey
  ): Promise<TransactionInstruction> {
    const { amount, claimant, index, proof } = args;
    const [claimStatus, bump] = await findClaimStatusKey(index, this.key);

    return this.program.instruction.claim(
      bump,
      index,
      amount,
      proof.map((p) => toBytes32Array(p)),
      {
        accounts: {
          distributor: this.key,
          claimStatus,
          from: this.distributorATA,
          to: await getATAAddress({ mint: this.data.mint, owner: claimant }),
          claimant,
          payer,
          systemProgram: SystemProgram.programId,
          tokenProgram: TOKEN_PROGRAM_ID,
        },
      }
    );
  }

  async claim(args: ClaimArgs): Promise<TransactionEnvelope> {
    const { provider } = this.sdk;
    const tx = new TransactionEnvelope(provider, [
      await this.claimIX(args, provider.wallet.publicKey),
    ]);
    const { instruction } = await getOrCreateATA({
      provider,
      mint: this.data.mint,
      owner: args.claimant,
    });
    if (instruction) {
      tx.instructions.unshift(instruction);
    }
    // check data.mint is wrapped sol or not
    // if wrapped sol, add unwrap instruction
    if (this.data.mint.equals(NATIVE_MINT)) {
      const tokenAccount = await Token.getAssociatedTokenAddress(
        ASSOCIATED_TOKEN_PROGRAM_ID,
        TOKEN_PROGRAM_ID,
        NATIVE_MINT,
        provider.wallet.publicKey
      );

      const unwrapIx = Token.createCloseAccountInstruction(
        TOKEN_PROGRAM_ID,
        tokenAccount,
        provider.wallet.publicKey,
        provider.wallet.publicKey,
        []
      );
      tx.instructions.push(unwrapIx);
    }

    return tx;
  }

  async getClaimStatus(index: u64): Promise<ClaimStatus> {
    const [key] = await findClaimStatusKey(index, this.key);
    return this.program.account.claimStatus.fetch(key);
  }

  async reload(): Promise<void> {
    this.data = await this.program.account.merkleDistributor.fetch(this.key);
  }
}

==> merkle-airdrop-0.3.6.tgz: package/src/wrapper.ts <==
import { TransactionEnvelope } from "@saberhq/solana-contrib";
import {
  ASSOCIATED_TOKEN_PROGRAM_ID,
  getATAAddress,
  getOrCreateATA,
  NATIVE_MINT,
  TOKEN_PROGRAM_ID,
  u64,
} from "@saberhq/token-utils";
import { Token } from "@solana/spl-token";
import type { PublicKey, TransactionInstruction } from "@solana/web3.js";
import { SystemProgram } from "@solana/web3.js";

import { findClaimStatusKey, findDistributorKey } from "./pda";
import type { MerkleDistributorSDK } from "./sdk";
import type {
  ClaimArgs,
  ClaimStatus,
  CreateAirdropArgs,
  CreateDistributorArgs,
  DistributorData,
  MerkleDistributorProgram,
  PendingDistributor,
} from "./types";
import { parseBalanceMap, toBytes32Array } from "./utils";

export class MerkleDistributorWrapper {
  readonly program: MerkleDistributorProgram;
  readonly key: PublicKey;
  readonly distributorATA: PublicKey;
  data: DistributorData;

  constructor(
    readonly sdk: MerkleDistributorSDK,
    key: PublicKey,
    distributorATA: PublicKey,
    data: DistributorData
  ) {
    this.program = sdk.program;
    this.key = key;
    this.distributorATA = distributorATA;
    this.data = data;
  }

  static async load(
    sdk: MerkleDistributorSDK,
    key: PublicKey
  ): Promise<MerkleDistributorWrapper> {
    const data = await sdk.program.account.merkleDistributor.fetch(key);
    return new MerkleDistributorWrapper(
      sdk,
      key,
      await getATAAddress({ mint: data.mint, owner: key }),
      data
    );
  }

  static async createMerkleAirdrop(
    args: CreateAirdropArgs
  ): Promise<PendingDistributor> {
    const { airdropList, airdropName, tokenMint, sdk } = args;

    const balanceMap: { [authority: string]: u64 } = {};
    airdropList.forEach(({ authority, amount }) => {
      const prevBalance = balanceMap[authority];
      if (prevBalance) {
        balanceMap[authority] = prevBalance.add(new u64(amount));
      } else {
        balanceMap[authority] = new u64(amount);
      }
    });

    const { claims, merkleRoot, tokenTotal } = parseBalanceMap(
      Object.entries(balanceMap).map(([authority, amount]) => ({
        address: authority,
        earnings: amount.toString(),
      }))
    );

    const pendingDistributor = await this.createDistributor({
      sdk,
      root: merkleRoot,
      maxTotalClaim: new u64(tokenTotal),
      maxNumNodes: new u64(Object.keys(claims).length),
      tokenMint: tokenMint ? tokenMint : NATIVE_MINT,
      name: airdropName,
      withdrawAuthority: sdk.provider.wallet.publicKey,
    });
    return pendingDistributor;
  }

  static async createDistributor(
    args: CreateDistributorArgs
  ): Promise<PendingDistributor> {
    const { root, tokenMint, name, withdrawAuthority } = args;

    // Validate the name must be less than 32 bytes
    if (Buffer.from(name).length > 32) {
      throw new Error("Name must be less than 32 bytes");
    }

    const { sdk } = args;
    const { provider } = sdk;

    const [distributor, bump] = await findDistributorKey(
      provider.wallet.publicKey,
      name
    );

    const ixs: TransactionInstruction[] = [];
    ixs.push(
      sdk.program.instruction.newDistributor(
        bump,
        name,
        toBytes32Array(root),
        args.maxTotalClaim,
        args.maxNumNodes,
        withdrawAuthority,
        {
          accounts: {
            base: provider.wallet.publicKey,
            distributor,
            mint: tokenMint,
            payer: provider.wallet.publicKey,
            systemProgram: SystemProgram.programId,
          },
        }
      )
    );

    const { address, instruction } = await getOrCreateATA({
      provider,
      mint: tokenMint,
      owner: distributor,
    });
    if (instruction) {
      ixs.push(instruction);
    }

    return {
      creator: provider.wallet.publicKey,
      name,
      bump,
      distributor,
      distributorATA: address,
      root: root.toString("hex"),
      tx: new TransactionEnvelope(provider, ixs),
    };
  }

  async claimIX(
    args: ClaimArgs,
    payer: PublicKey
  ): Promise<TransactionInstruction> {
    const { amount, claimant, index, proof } = args;
    const [claimStatus, bump] = await findClaimStatusKey(index, this.key);

    return this.program.instruction.claim(
      bump,
      index,
      amount,
      proof.map((p) => toBytes32Array(p)),
      {
        accounts: {
          distributor: this.key,
          claimStatus,
          from: this.distributorATA,
          to: await getATAAddress({ mint: this.data.mint, owner: claimant }),
          claimant,
          payer,
          systemProgram: SystemProgram.programId,
          tokenProgram: TOKEN_PROGRAM_ID,
        },
      }
    );
  }

  async claim(args: ClaimArgs): Promise<TransactionEnvelope> {
    const { provider } = this.sdk;
    const tx = new TransactionEnvelope(provider, [
      await this.claimIX(args, provider.wallet.publicKey),
    ]);
    const { instruction } = await getOrCreateATA({
      provider,
      mint: this.data.mint,
      owner: args.claimant,
    });
    if (instruction) {
      tx.instructions.unshift(instruction);
    }
    // check data.mint is wrapped sol or not
    // if wrapped sol, add unwrap instruction
    if (this.data.mint.equals(NATIVE_MINT)) {
      const tokenAccount = await Token.getAssociatedTokenAddress(
        ASSOCIATED_TOKEN_PROGRAM_ID,
        TOKEN_PROGRAM_ID,
        NATIVE_MINT,
        provider.wallet.publicKey
      );

      const unwrapIx = Token.createCloseAccountInstruction(
        TOKEN_PROGRAM_ID,
        tokenAccount,
        provider.wallet.publicKey,
        provider.wallet.publicKey,
        []
      );
      tx.instructions.push(unwrapIx);
    }

    return tx;
  }

  async getClaimStatus(index: u64): Promise<ClaimStatus> {
    const [key] = await findClaimStatusKey(index, this.key);
    return this.program.account.claimStatus.fetch(key);
  }

  async reload(): Promise<void> {
    this.data = await this.program.account.merkleDistributor.fetch(this.key);
  }
}
